<?php

namespace Drupal\static_asset_cache_buster\Plugin\Linkit\Substitution;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\linkit\SubstitutionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Attempts to add a cache buster to generated URLs.
 *
 * @Substitution(
 *   id = "static_asset_cache_buster",
 *   label = @Translation("Static Asset Cache Buster"),
 * )
 */
class StaticAssetCacheBuster extends PluginBase implements SubstitutionInterface, ContainerFactoryPluginInterface {

  /**
   * An unaltered instance of the overridden plugin.
   *
   * @var \Drupal\linkit\SubstitutionInterface
   */
  protected $parentPlugin;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    // Recontruct the overridden plugin.
    $plugin_definition = $this->pluginDefinition;
    $original_class = $plugin_definition['original_class'];
    $plugin_definition['class'] = $plugin_definition['original_class'];
    unset($plugin_definition['original_class']);
    $this->parentPlugin = $original_class::create(\Drupal::getContainer(), [], $this->pluginId, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getUrl(EntityInterface $entity) {
    /** @var \Drupal\Core\GeneratedUrl $url */
    $generated_url = $this->parentPlugin->getUrl($entity);
    $url = $generated_url->getGeneratedUrl();

    /** @var \Drupal\Core\StreamWrapper\StreamWrapperInterface $stream_wrapper_manager */
    $stream_wrapper_manager = \Drupal::service('stream_wrapper_manager');
    $wrappers = $stream_wrapper_manager->getWrappers();
    // Loop through stream wrappers and attempt to match the absolute URL with
    // a stream wrapper.
    foreach ($wrappers as $id => $wrapper) {
      $wrapper_base_url = $wrapper['class']::baseUrl();

      if (str_starts_with($url, $wrapper_base_url)) {
        $uri = str_replace("$wrapper_base_url/", "$id://", $url);

        /** @var \Drupal\file\FileInterface[] $files */
        $files = \Drupal::entityTypeManager()
          ->getStorage('file')
          ->loadByProperties(['uri' => $uri]);
        /** @var \Drupal\file\FileInterface|null $file */
        $file = reset($files) ?: NULL;

        if ($file) {
          $changed = $file->getChangedTime();
          $timestamp_query = _static_asset_cache_buster_get_cache_buster_query($changed);
          $url .= (strpos($url, '?') !== FALSE ? '&' : '?') . UrlHelper::buildQuery($timestamp_query);
          $generated_url->setGeneratedUrl($url);
        }

        break;
      }
    }

    return $generated_url;
  }

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(EntityTypeInterface $entity_type) {
    // Because the 'static_asset_cache_buster' plugin is removed by
    // static_asset_cache_buster_linkit_substitution_alter(), this method will
    // never be called, so the return value is irrelevant.
    return FALSE;
  }

}
