<?php

namespace Drupal\static_asset_cache_buster;

use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\linkit\SubstitutionManager as SubstitutionManagerUpstream;

/**
 * A plugin manager for the substitution plugins.
 */
class SubstitutionManager extends SubstitutionManagerUpstream {

  /**
   * Filter the list of plugins by their applicability.
   *
   * @param array $definitions
   *   An array of plugin definitions.
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type to get applicable plugins for.
   *
   * @return array
   *   The definitions appropriate for the given entity type.
   *
   * @see SubstitutionInterface::isApplicable()
   */
  protected function filterPlugins(array $definitions, EntityTypeInterface $entity_type) {
    return array_filter($definitions, function ($definition) use ($entity_type) {
      // Use original class, which is added by
      // static_asset_cache_buster_linkit_substitution_alter().
      /** @var \Drupal\linkit\SubstitutionInterface $class */
      $class = $definition['original_class'];
      return $class::isApplicable($entity_type);
    });
  }

}
